Package: immunotation
Type: Package
Title: Tools for working with diverse immune genes
Version: 1.20.0
Date: 2021-08-09
Authors@R: c(
    person("Katharina", "Imkeller", email = "ag.imkeller@gmail.com", role = c("cre", "aut")))
Description: MHC (major histocompatibility complex) molecules are cell surface complexes that present antigens to T cells.  The repertoire of antigens presented in a given genetic background largely depends on the sequence of the encoded MHC molecules, and thus, in humans, on the highly variable HLA (human leukocyte antigen) genes of the hyperpolymorphic HLA locus. More than 28,000 different HLA alleles have been reported, with significant differences in allele frequencies between human populations worldwide. Reproducible and consistent annotation of HLA alleles in large-scale bioinformatics workflows remains challenging, because the available reference databases and software tools often use different HLA naming schemes. The package immunotation provides tools for consistent annotation of HLA genes in typical immunoinformatics workflows such as for example the prediction of MHC-presented peptides in different human donors. Converter functions that provide mappings between different HLA naming schemes are based on the MHC restriction ontology (MRO). The package also provides automated access to HLA alleles frequencies in worldwide human reference populations stored in the Allele Frequency Net Database.
License: GPL-3
Encoding: UTF-8
Collate: 'external_resources_input.R' 'AFND_interface.R'
        'MRO_interface_helper.R' 'MRO_interface.R' 'MACUI_interface.R'
        'nomenclature_queries.R' 'AFND_queries.R' 'visualization.R'
Depends: R (>= 4.1)
Imports: stringr, ontologyIndex, curl, ggplot2, readr, rvest, tidyr,
        xml2, maps, rlang
Suggests: BiocGenerics, rmarkdown, BiocStyle, knitr, testthat, DT
VignetteBuilder: knitr
RoxygenNote: 7.1.1
biocViews: Software, ImmunoOncology, BiomedicalInformatics, Genetics,
        Annotation
BugReports: https://github.com/imkeller/immunotation/issues
git_url: https://git.bioconductor.org/packages/immunotation
git_branch: RELEASE_3_23
git_last_commit: 0725d76
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:49:21 UTC; biocbuild
Author: Katharina Imkeller [cre, aut]
Maintainer: Katharina Imkeller <ag.imkeller@gmail.com>
