% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/external_resources_input.R
\name{load_mro}
\alias{load_mro}
\title{load_mro}
\usage{
load_mro()
}
\value{
MRO in ontology_index format
}
\description{
load_mro
}
\keyword{internal}
